clear all
close all
%% Parameters
l_t1 = 256; % t1 full grid
l_t2 = 1024; % t2 full grid
l_tau = 10; % tau full grid

% 2D FID is constructed by correlating 1D FIDs:

a1 = [100, 80, 150]; % F1 peak amplitudes
w1 = [50, 52, 70];   % F1 frequencies
r1 = 0*[0.5, 0.5, 0.5];  % F1 decay rates

a2 = [70, 150, 350]; % F2 peak amplitudes
w2 = [250, 270, 320]; % F2 frequencies
r2 = [1, 3, 2]; % F2 decay rates

corr = [[1, 0, 0];
        [1, 0, 1];
        [0, 1, 0]]; % 1D peak correlations: 1-1, 1-2, 1-3, ... 

p3_exp = [[0.6, 0.5, 0.5]; 
          [0.0, 0.1, 0.0]]; % exponential decay in tau pseudo-dim for F1 and F2
      
% p3_lin = [[0, 0, 0]; 
%           [0, 0, 0]]; % linear decay in tau pseudo-dim for F1 and F2

shift_rate1 = 0*[1, 1, 1]; % rate of peaks shift in F1
shift_rate2 = 0*[1, 1, 1]; % rate of peaks shift in F2

noise_level = 400; % scale noise
%% Generating grid
t1 = linspace(0, l_t1 - 1, l_t1)*2*pi/l_t1; % t1 vector, always from 0 to 1*2pi
t2 = linspace(0, l_t2 - 1, l_t2)*2*pi/l_t2; % t2 vector, always from 0 to 1*2pi
tau = linspace(0, l_tau - 1, l_tau); % tau vector
%% Generating 3D noise for conventional series
seed = 1; % for reproducible noise in every script execution
rng(seed);
noise = randn(2*length(t1), length(t2), length(tau)) + 1j*randn(2*length(t1), length(t2), length(tau));
noise = noise*noise_level;
%% 3D series - conventional
a1_exp = zeros(l_tau, length(a1));
a2_exp = zeros(l_tau, length(a2));
for i=1:length(tau)
   a1_exp(i, :) = a1.*exp(-p3_exp(1, :)*tau(i)); % define amplitude change with yau
   a2_exp(i, :) = a2.*exp(-p3_exp(2, :)*tau(i));
   w1_shift = w1 + i* shift_rate1;               % define peak shift in tau
   w2_shift = w2 + i* shift_rate2; 
   [f1_cos, f1_sin, s1] = create_f1(a1_exp(i, :), w1_shift, r1, t1);  % create 1D FID (t1)
   [f2, s2] = create_f2(a2_exp(i, :), w2_shift, r2, t2);              % create 1D FID (t2) 
   [F_2d, S_2d, F_cos_2d, F_sin_2d] = create_2D(f1_cos, f1_sin, f2, corr, squeeze(noise(:, :, i))); % create 2D FID 
   %
   F_3d_exp(:, :, i) = F_2d;
   S_3d_exp(:, :, i) = S_2d;
   F_cos_3d_exp(:, :, i) = F_cos_2d;
   F_sin_3d_exp(:, :, i) = F_sin_2d;
end   

for i=1:length(tau)
    plot_contours(real(squeeze(S_3d_exp(:, :, i))),  max(max(real(squeeze(S_3d_exp(:, :, i)))))/20, 20); title(num2str(i)); xlim([240 340]); ylim([45 75])
end
%% Generating 3D noise for accordion
seed = 1; % for reproducible noise (the same noise as in conventional - for different, change seed)
rng(seed);
noise2 = randn(2*length(t1), length(t2), length(tau)) + 1j*randn(2*length(t1), length(t2), length(tau));
noise2 = noise2*noise_level/sqrt(l_tau); % divide by sqrt(l_tau) to reflect the same "experimental time" as in series
%% Accordion
for i=1:length(tau)         % Generation of 2D FID - the same as in conventional series, different noise
   a1_exp(i, :) = a1.*exp(-p3_exp(1, :)*tau(i));
   a2_exp(i, :) = a2.*exp(-p3_exp(2, :)*tau(i));
   w1_shift = w1 + i* shift_rate1; 
   w2_shift = w2 + i* shift_rate2; 
   [f1_cos, f1_sin, s1] = create_f1(a1_exp(i, :), w1_shift, r1, t1);
   [f2, s2] = create_f2(a2_exp(i, :), w2_shift, r2, t2);
   [F_2d, S_2d, F_cos_2d, F_sin_2d] = create_2D(f1_cos, f1_sin, f2, corr, squeeze(noise2(:, :, i)));
   F_3d_exp(:, :, i) = F_2d;
   S_3d_exp(:, :, i) = S_2d;
   F_cos_3d_exp(:, :, i) = F_cos_2d;
   F_sin_3d_exp(:, :, i) = F_sin_2d;
end   


F_cos_3d_exp_radial = zeros(l_t1, l_t2); % pseudo-2D FID for accordion sampling
F_sin_3d_exp_radial = zeros(l_t1, l_t2);
  
for i = 1:l_t2
    X = squeeze(F_cos_3d_exp(:, i, :));
    F_cos_3d_exp_radial(:, i) = interpolation(X); % interpolate conventional sampling grid in t2 for radial sampling
    X = squeeze(F_sin_3d_exp(:, i, :));
    F_sin_3d_exp_radial(:, i) = interpolation(X);
end

S_cos_2d_exp_radial = real(fft(F_cos_3d_exp_radial, [], 2));
S_sin_2d_exp_radial = real(fft(F_sin_3d_exp_radial, [], 2));
S_2d_exp_radial = fft(S_cos_2d_exp_radial + 1i*S_sin_2d_exp_radial, [], 1);

sp_radial = real(squeeze(S_2d_exp_radial));
plot_contours(sp_radial,  max(max(sp_radial))/20, 20); title('Accordion spectrum'); xlim([240 340]); ylim([45 75])
