% Runs l1-constrained FT-ILT processing of data with joint NUS of t1/tau
% Processing is time-consuming (up to hours)

clear all
close all
%% Parameters
FTILT_points = 256
Iterations=1000% For fast and robust calculations
%Iterations=10000% Long calculation. Value used in publication

t1 = linspace(0, 0.255, 64); % t1 vector
t2 = linspace(0, 1.023, 256); % t2 vector
tau = linspace(0, 49, 50); % tau vector

% 2D FID is constructed by correlating 1D FIDs:

a1 = [120, 120, 190]; % F1 peak amplitudes
w1 = [5120, 5120, 10000];   % F1 frequencies
r1 = 0*[0.5, 0.5, 0.5];  % F1 decay rates

a2 = [175, 175, 160]; % F2 peak amplitudes
w2 = [500, 500, 2660]; % F2 frequencies
r2 = [10, 35, 25]; % F2 decay rates

corr = [[1, 0, 0];
        [0, 1, 0];
        [0, 0, 1]]; % 1D peak correlations: 1-1, 1-2, 1-3, ... 

p3_exp = [[2e-2, 1e-1, 3e-2]; 
          [2e-2, 1e-1, 3e-2]]; % exponential decay in tau pseudo-dim for F1 and F2
      
noise_level = 300; % scale noise
seed = 123; % for reproducible noise and schedule in every script execution
%% Generating 3D noise
rng(seed);
noise = randn(2*length(t1), length(t2), length(tau)) + 1j*randn(2*length(t1), length(t2), length(tau));
noise = noise*noise_level; 
%% 3D series - exponential decay
for i=1:length(tau)
   a1_exp = a1.*exp(-p3_exp(1, :)*tau(i));
   a2_exp = a2.*exp(-p3_exp(2, :)*tau(i));
   [f1_cos, f1_sin, s1] = create_f1(a1_exp, w1, r1, t1);
   [f2, s2] = create_f2(a2_exp, w2, r2, t2);
   [F_2d, S_2d, F_cos_2d, F_sin_2d] = create_2D(f1_cos, f1_sin, f2, corr, squeeze(noise(:, :, i)));
   F_3d_exp(:, :, i) = F_2d;
   S_3d_exp(:, :, i) = S_2d;
   F_cos_3d_exp(:, :, i) = F_cos_2d;
   F_sin_3d_exp(:, :, i) = F_sin_2d;
end   
FullFid=F_3d_exp;
FullSpectrum=real(S_3d_exp);
schedf=randi(size(FullFid,1)/2,FTILT_points,1);
    schedD=randi(size(FullFid,3),FTILT_points,1);
schedf(1)=1;
schedD(1)=1;
FID=zeros(size(FullFid,2),FTILT_points);
for i=1:FTILT_points
  FID(:,i)=real(fftshift(fft(squeeze(FullFid(2*schedf(i)-1,:,schedD(i))))))+real(fftshift(fft(squeeze(FullFid(2*schedf(i),:,schedD(i))))))*1i;   
end
sw=100
sched=[schedf-1,tau(schedD)'];
d_grid=logspace(-2,-0.5,60);

% FTILT reconstruction: solves problem described by formula (8) in paper
%Shchukina et al., "Alternative data processing techniques for serial NMR
%experiments", Concepts in Magnetic Resonance
SPECTRUM=ITAMED3D(FID, sched,sw, 128, d_grid,Iterations,100);
SPECTRUM=fftshift(fftshift(SPECTRUM,1),2);
%%% PLOTTING RESULTS
ClassicDecay=real(squeeze(FullSpectrum(39,350,[1,25])))
ClassicTau=tau([1,25])
DecayRateClassic(1,:)=polyfit(ClassicTau',log(ClassicDecay),1)
ClassicDecay=real(squeeze(FullSpectrum(58,156,[1,25])))
ClassicTau=tau([1,25])
DecayRateClassic(2,:)=polyfit(ClassicTau',log(ClassicDecay),1)
f1=linspace(1,128,size(SPECTRUM,2));
f2=linspace(1,128,size(SPECTRUM,1));
f1O=linspace(1,128,size(FullSpectrum,1));
f2O=linspace(1,128,size(FullSpectrum,2));
Omax=max(max(real(FullSpectrum(:,:,1))));
Smax=max(max(sum(SPECTRUM,3)));
conS3=linspace(Smax/25,Smax,40);
con0=linspace(Omax/25,Omax,40);
Fontsize1=25;
Fontsize2=14
Linewidth=2;
figure('rend','painters','pos',[10 10 1400 600])
FigureSubplot(1)=subplot(1,3,1)
contour(f1,f2,squeeze(max(SPECTRUM,[],3)),conS3, 'LineWidth',Linewidth)
set(gca,'YTickLabel',[])
set(gca,'XTickLabel',[])
text(57,39,' \leftarrow c)','FontSize',Fontsize2)
text(39,87,' \leftarrow b)','FontSize',Fontsize2)
text(4,128*0.95,'a)','FontSize',Fontsize1)
xlabel('F2','FontSize',Fontsize1)
ylabel('F1','FontSize',Fontsize1)
FigureSubplot(2)=subplot(2,3,2)
semilogx(d_grid,squeeze(SPECTRUM(78,57,:)),'LineWidth',Linewidth)% contour(squeeze(max(SPECTRUM,[],2)),conS2)
set(gca,'YTickLabel',[])
[a,b]=max(squeeze(SPECTRUM(78,57,:)));
text(d_grid(b),a,[' \leftarrow ',num2str(d_grid(b),'% 1.4f')],'FontSize',Fontsize2)
text(d_grid(2),a*0.95,'b)','FontSize',Fontsize1)
set(gca,'XTickLabel',[])
FigureSubplot(3)=subplot(2,3,5)
semilogx(d_grid,squeeze(SPECTRUM(175,38,:)),'LineWidth',Linewidth)
[a,b]=max(squeeze(SPECTRUM(175,38,:)));
text(d_grid(2),a*0.95,'c)','FontSize',Fontsize1)
text(d_grid(b),a*0.99,[' \leftarrow ',num2str(d_grid(b),'% 1.4f')],'FontSize',Fontsize2)
[a,b]=max(squeeze(SPECTRUM(175,38,40:end)));
text(d_grid(b+40),a*0.85,[' \leftarrow ',num2str(d_grid(b+40),'% 1.4f')],'FontSize',Fontsize2)
set(gca,'YTickLabel',[])
xlabel('Decay rate in \tau','FontSize',Fontsize2)
FigureSubplot(4)=subplot(2,3,3)
contour(f1O,f2O,squeeze(FullSpectrum(:,:,1))',con0,'LineWidth',Linewidth)
set(gca,'YTickLabel',[])
set(gca,'XTickLabel',[])
ylabel('F1','FontSize',Fontsize2)
text(4,128*0.9,'d)','FontSize',Fontsize1)
FigureSubplot(5)=subplot(2,3,6)
contour(f1O,f2O,squeeze(FullSpectrum(:,:,25))',con0,'LineWidth',Linewidth)
set(gca,'YTickLabel',[])
set(gca,'XTickLabel',[])
text(4,128*0.9,'e)','FontSize',Fontsize1)
xlabel('F2','FontSize',Fontsize2)
ylabel('F1','FontSize',Fontsize2)
text(57,40,[' \leftarrow ',num2str(-DecayRateClassic(1,1),'% 1.4f')],'FontSize',Fontsize2)
text(39,88,[' \leftarrow ',num2str(-DecayRateClassic(2,1),'% 1.4f')],'FontSize',Fontsize2)
pos = get(FigureSubplot, 'Position' );
pos{1}(1)=0.06;
pos{2}(1)= pos{1}(1)+pos{1}(3)+0.01
pos{3}(1)= pos{1}(1)+pos{1}(3)+0.01;
pos{2}(3)=pos{2}(3)+0.2;
pos{3}(3)=pos{3}(3)+0.2;
pos{4}(1)= pos{1}(1)+pos{1}(3)+pos{2}(3)+0.04;
pos{5}(1)= pos{1}(1)+pos{1}(3)+pos{2}(3)+0.04;
pos{2}(2)= pos{2}(2)+0.01;
pos{2}(4)= pos{2}(4)-0.01;
pos{3}(2)= pos{3}(2);
pos{3}(4)= pos{3}(4)-0.01;
pos{4}(2)= pos{4}(2)+0.01;
pos{4}(4)= pos{4}(4)-0.01;
pos{5}(2)= pos{1}(2);
pos{5}(4)=pos{1}(4)/2;
pos{4}(4)=pos{1}(4)/2;
pos{4}(2)=pos{1}(2)+pos{1}(4)/2;
pos{3}(2)= pos{1}(2); 
pos{3}(4)=pos{1}(4)/2;
pos{2}(4)=pos{1}(4)/2;
pos{2}(2)=pos{1}(2)+pos{1}(4)/2;
for i=1:5
set(FigureSubplot(i), 'Position', pos{i} ) ;
end
