% SCRIPT REQUIRES MDDNMR software installed (mddnmr.spektrino.com -> Downloads)
% Runs MDD processing of a series of conventionally sampled spectra

clear all
close all
%% Parameters
t1 = linspace(0, 0.255, 64); % t1 vector 
t2 = linspace(0, 1.023, 256); % t2 vector
tau = linspace(0, 49, 32); % tau vector

% 2D FID is constructed by correlating 1D FIDs:

a1 = [120, 120, 190]; % F1 peak amplitudes
w1 = [5000, 5050, 10000]; % F1 frequencies
r1 = [40, 35, 50];  % F1 decay rates

a2 = [175, 175, 160]; % F2 peak amplitudes
w2 = [450, 420, 2660]; % F2 frequencies
r2 = [30, 10, 0]; % F2 decay rates

corr = [[1, 0, 0];
        [0, 1, 0];
        [0, 0, 0]]; % 1D peak correlations: 1-1, 1-2, 1-3, ... 

p3_exp = [[2e-2, 2e-1, 3e-2]; 
          [2e-2, 2e-1, 3e-2]]; % exponential decay in tau pseudo-dim for F1 and F2
      
noise_level = 300; % scale noise
seed = 123; % for reproducible noise in every script execution
%% Generating 3D noise
rng(seed);
noise = randn(2*length(t1), length(t2), length(tau)) + 1j*randn(2*length(t1), length(t2), length(tau));
noise = noise*noise_level; 
%% 3D series - exponential decay
for i=1:length(tau)
   a1_exp = a1.*exp(-p3_exp(1, :)*tau(i));
   a2_exp = a2.*exp(-p3_exp(2, :)*tau(i));
   [f1_cos, f1_sin, s1] = create_f1(a1_exp, w1, r1, t1);
   [f2, s2] = create_f2(a2_exp, w2, r2, t2);
   [F_2d, S_2d, F_cos_2d, F_sin_2d] = create_2D(f1_cos, f1_sin, f2, corr, squeeze(noise(:, :, i)));
   F_3d_exp(:, :, i) = F_2d;
   S_3d_exp(:, :, i) = S_2d;
   F_cos_3d_exp(:, :, i) = F_cos_2d;
   F_sin_3d_exp(:, :, i) = F_sin_2d;
end   
FullFid=F_3d_exp;
FullSpectrum=S_3d_exp;
FID=reshape(FullFid,size(FullFid,1)*size(FullFid,2)*size(FullFid,3),1);
w=0;
FID2=[];
for l=1:size(F_3d_exp,3)
   for i=1:size(F_3d_exp,1)
       FID2=[FID2,real(squeeze(F_3d_exp(i,:,l))),imag(squeeze(F_3d_exp(i,:,l)))];      
   end
end
FIDfile=fopen('bin/Binary.bin','w');
fwrite(FIDfile, FID2, 'float');
fclose(FIDfile); 
cd bin
system('chmod a+x *')
system('./tomdd.com')
system('./tomdd2.com')
system('./proc.sh')
cd ..
NumberShapes=2;
f=fopen('bin/SHAPES/sh_X_01.ft1','r')
X=fread(f,'float')
X=X(513:end);
X=reshape(X,NumberShapes,512)
X=X(:,1:2:end);
X=(abs(X'));
fclose(f)
f=fopen('bin/SHAPES/sh_Z_01.ft1','r')
Z=fread(f,'float')
Z=Z(513:end);
Z=reshape(Z,NumberShapes,64)
Z=Z(:,1:2:end);
Z=(abs(Z'));
fclose(f)
Color1='blue';
Color2='red'
f=fopen('bin/SHAPES/sh_Y_01.ft1','r')
Y=fread(f,'float')
Y=Y(513:end);
Y=reshape(Y,NumberShapes,128)
Y=Y(:,2:2:end);
Y=Y(:,1:2:end)+ 1i*Y(:,2:2:end);
Y=(abs(real(fft(Y',128))));
fclose(f)
X1=flipud(X);
Linewidth=3
Fontsize=25
[acx,XConmax]=max((squeeze(max(FullSpectrum(:,:,1),[],1))));
[acy,YConmax]=max((squeeze(max(FullSpectrum(:,:,1),[],2))));
[ay,Xymax]=max(Y(:,1))
[az,Zmax]=max(Z(:,1))
[ax,Xxmax]=max(X1(:,1))
figure('rend','painters','pos',[10 10 900 800])
FigureSubplot(1)=subplot(2,2,1)
plot(X1(:,1),'LineWidth',Linewidth,'Color',Color1)
hold all
plot(X1(:,2),'LineWidth',Linewidth,'Color',Color2)
xlim([Xxmax-30,Xxmax+30])
set(gca,'YTickLabel',[])
set(gca,'XTick',[])
text(Xxmax-30,ax*1.05,' a)', 'FontSize',Fontsize)
set(gca,'XTickLabel',[])
FigureSubplot(2)=subplot(2,2,2)
plot(Z(:,1)./az,'LineWidth',Linewidth, 'Color',Color1)
hold all
plot(Z(:,2)./az,'LineWidth',Linewidth,'Color',Color2);
set(gca,'YTickLabel',[])
set(gca,'XTickLabel',[])
plot(real(squeeze((FullSpectrum(YConmax,XConmax,:)/acx))),'--','LineWidth',Linewidth,'Color',Color1);
plot(real(squeeze((FullSpectrum(30,366,:)/acx))),'--','LineWidth',Linewidth,'Color',Color2);
text(0,1.1,' b)', 'FontSize',Fontsize)
xlabel('\tau','FontSize',Fontsize)
FigureSubplot(3)=subplot(2,2,3)
set(gca,'YTickLabel',[])
set(gca,'XTickLabel',[])
contour(squeeze((real(FullSpectrum(:,:,1)))),[14],'LineWidth',Linewidth)
xlim([XConmax-30,XConmax+30])
ylim([YConmax-20,YConmax+20])
text(XConmax-30,YConmax+20-2,' c)', 'FontSize',Fontsize)
hold all
plot(366,30,'x','MarkerSize',Fontsize,'LineWidth',Linewidth, 'Color',Color2)
plot(XConmax,YConmax,'x','MarkerSize',Fontsize,'LineWidth',Linewidth, 'Color',Color1)
set(gca,'YTickLabel',[])
set(gca,'XTickLabel',[])
xlabel('F2','FontSize',Fontsize)
ylabel('F1','FontSize',Fontsize)
FigureSubplot(4)=subplot(2,2,4)
plot(Y(:,1)./az,'LineWidth',Linewidth, 'Color',Color1)
hold all
plot(Y(:,2)./az,'LineWidth',Linewidth,'Color',Color2);
xlim([Xymax-20,Xymax+20])
set(gca,'YTickLabel',[])
set(gca,'XTickLabel',[])
set(gca,'XTick',[])
text(Xymax-18,0,' d)', 'FontSize',Fontsize)
camroll(-90)
pos = get(FigureSubplot, 'Position' );
pos{1}(1)=0.06;
pos{1}(2)=0.52;
pos{1}(3)=0.45;
pos{1}(4)=0.45;
pos{2}(4)=0.45;
pos{3}(4)=0.45;
pos{4}(4)=0.45;
pos{3}(3)=0.45;
pos{4}(3)=0.45;
pos{3}(1)=pos{1}(1);
pos{3}(2)=pos{1}(2)-pos{1}(4);
pos{4}(2)=pos{1}(2)-pos{1}(4);
pos{4}(1)=pos{1}(1)+pos{1}(3);
pos{2}(1)=pos{1}(1)+pos{1}(3)+0.02;
pos{2}(2)=pos{1}(2)+0.05;
pos{2}(3)=pos{1}(3)-0.02;
pos{2}(4)=pos{1}(4)-0.05;
for i=1:4
set(FigureSubplot(i), 'Position', pos{i} );
end


