clear all
close all
%% Parameters
l_t1 = 64; % t1 full grid
l_t2 = 1024; % t2 full grid
l_tau = 64; % tau full grid

% 2D FID is constructed by correlating 1D FIDs:

a1 = [100, 100, 100]; % F1 peak amplitudes
w1 = [50, 55, 40]; % F1 frequencies
r1 = [0.5, 0.5, 0.5]; % F1 decay rates


a2 = [100, 100, 100]; % F1 peak amplitudes
w2 = [250, 270, 340]; % F1 frequencies
r2 = [1, 3, 2]; % F1 decay rates

corr = [[1, 0, 0];
        [1, 0, 1]; 
        [0, 1, 0]]; % 1D peak correlations: 1-1, 1-2, 1-3, ... 

p3_exp = [[0, 0, 0]; 
          [0, 0, 0]]; % exponential decay in tau pseudo-dim for F1 and F2
      
% p3_lin = [[0, 0, 0]; 
%           [0, 0, 0]]; % linear decay in tau pseudo-dim for F1 and F2

shift_rate1 = 0*[1, 1, 1]; % rate of peaks shift in F1
shift_rate2 = [0.2, 0.4, -0.4]; % % rate of peaks shift in F2

noise_level = 5000; % scale noise
%% Generating grid
t1 = linspace(0, l_t1 - 1, l_t1)*2*pi/l_t1; % t1 vector, always from 0 to 1*2pi
t2 = linspace(0, l_t2 - 1, l_t2)*2*pi/l_t2; % t2 vector, always from 0 to 1*2pi
tau = linspace(0, l_tau - 1, l_tau); % tau vector
%% Generating 3D noise for conventional series
seed = 1; % for reproducible noise in every script execution
rng(seed);
noise = randn(2*length(t1), length(t2), length(tau)) + 1j*randn(2*length(t1), length(t2), length(tau));
noise = noise*noise_level;
%% 3D series - conventional
for i=1:l_tau
   a1_exp = a1.*exp(-p3_exp(1, :)*tau(i));
   a2_exp = a2.*exp(-p3_exp(2, :)*tau(i));
   w1_shift = w1 +i* shift_rate1; 
   w2_shift = w2 +i* shift_rate2; 
   [f1_cos, f1_sin, s1] = create_f1(a1_exp, w1_shift, r1, t1);
   [f2, s2] = create_f2(a2_exp, w2_shift, r2, t2);
   [F_2d, S_2d, F_cos_2d, F_sin_2d] = create_2D(f1_cos, f1_sin, f2, corr, squeeze(noise(:, :, i)));
   F_3d_exp(:, :, i) = F_2d;
   S_3d_exp(:, :, i) = S_2d;
   F_cos_3d_exp(:, :, i) = F_cos_2d;
   F_sin_3d_exp(:, :, i) = F_sin_2d;
end   

plot_contours(real(squeeze(S_3d_exp(:, :, 1))),  max(max(real(squeeze(S_3d_exp(:, :, 1)))))/15, 20); title('first 2D spectrum'); xlim([240 360]); ylim([35 65])
plot_contours(real(squeeze(S_3d_exp(:, :, l_tau))),  max(max(real(squeeze(S_3d_exp(:, :, l_tau)))))/15, 20); title('last 2D spectrum'); xlim([240 360]); ylim([35 65])
%% Generating 3D noise for Radon
seed = 1; % for reproducible noise (the same noise as in conventional - for different, change seed)
rng(seed);
noise2 = randn(2*length(t1), length(t2), length(tau)) + 1j*randn(2*length(t1), length(t2), length(tau));
noise2 = noise2*noise_level/sqrt(l_tau); % divide by sqrt(l_tau) to reflect the same "experimental time" as in series
%% Semi-Radon
for i=1:l_tau      % Generation of 2D FID - the same as in conventional series, different noise
   a1_exp = a1.*exp(-p3_exp(1, :)*tau(i));
   a2_exp = a2.*exp(-p3_exp(2, :)*tau(i));
   w1_shift = w1 +i* shift_rate1; 
   w2_shift = w2 +i* shift_rate2; 
   [f1_cos, f1_sin, s1] = create_f1(a1_exp, w1_shift, r1, t1);
   [f2, s2] = create_f2(a2_exp, w2_shift, r2, t2);
   [F_2d, S_2d, F_cos_2d, F_sin_2d] = create_2D(f1_cos, f1_sin, f2, corr, squeeze(noise2(:, :, i)));
   F_3d_exp(:, :, i) = F_2d;
   S_3d_exp(:, :, i) = S_2d;
   F_cos_3d_exp(:, :, i) = F_cos_2d;
   F_sin_3d_exp(:, :, i) = F_sin_2d;
end   

F_cos_3d_exp_radial = zeros(l_t1, l_t2);
F_sin_3d_exp_radial = zeros(l_t1, l_t2);

for i = 1:l_t2
    X = squeeze(F_cos_3d_exp(:, i, :));
    F_cos_3d_exp_radial(:, i) = interpolation(X);
    X = squeeze(F_sin_3d_exp(:, i, :));
    F_sin_3d_exp_radial(:, i) = interpolation(X);
end

S_cos_2d_exp_radial = real(fft(F_cos_3d_exp_radial, 2^nextpow2(2*size(F_cos_3d_exp_radial, 2)), 2));
S_sin_2d_exp_radial = real(fft(F_sin_3d_exp_radial, 2^nextpow2(2*size(F_sin_3d_exp_radial, 2)), 2));
S_2d_exp_radial = fft(S_cos_2d_exp_radial + 1i*S_sin_2d_exp_radial, [], 1);

u = exp(2*pi*1i/l_t2*0.2); % matching condition for two peaks - rate of shift=0.2

for i=1:l_t1
    for j = 1:l_t2
        shift_matrix(i,j) =  u^(-(i-1)*(j-1));
    end
end

S_cos_2d_exp_radial = real(fft(F_cos_3d_exp_radial.*shift_matrix, [], 2));
S_sin_2d_exp_radial = real(fft(F_sin_3d_exp_radial.*shift_matrix, [], 2));
S_2d_exp_radial = fft(S_cos_2d_exp_radial + 1i*S_sin_2d_exp_radial, [], 1);

plot_contours(real(squeeze(S_2d_exp_radial)), max(max(real(squeeze(S_2d_exp_radial))))/15, 20); title('semi-Radon'); xlim([240 360]); ylim([35 65])
%% Radon
for i =1:l_t1
    for k = 1:l_tau
        shift = exp(-1i*t2/5).^k;
        f_cos_3d_exp_aux(i, :, k) = F_cos_3d_exp(i, :, k).*shift;
        f_sin_3d_exp_aux(i, :, k) = F_sin_3d_exp(i, :, k).*shift;
    end
end

f_cos_3d_exp_rad = sum(f_cos_3d_exp_aux,3);
f_sin_3d_exp_rad = sum(f_sin_3d_exp_aux,3);

S_cos_2d_exp_rad = real(fft( f_cos_3d_exp_rad, [], 2));
S_sin_2d_exp_rad = real(fft(f_sin_3d_exp_rad, [], 2));
S_2d_exp_rad = fft(S_cos_2d_exp_rad + 1i*S_sin_2d_exp_rad, [], 1);

plot_contours(real(squeeze(S_2d_exp_rad)),  max(max(real(squeeze(S_2d_exp_rad))))/15, 20); title('Radon'); xlim([240 360]); ylim([35 65])
